<?php namespace EscapeWork\Manager\News\Models;

use EscapeWork\LaravelHelpers\Eloquent;
use Carbon\Carbon;
use InvalidArgumentException;

class News extends Eloquent
{

    /**
     * Tabela
     */
    protected $table = 'news';

    /**
     * Sluggable
     */
    protected $sluggable = true;

    /**
     * Dates array
     */
    protected $dates = array('date');

    /**
     * Regras de validação
     */
    public static $validationRules = array(
        'title'       => array('required'),
        'description' => array('required'),
        'date'        => array('required'),
    );

    /**
     * Mensagens de validação
     */
    public static $validationMessages = array(
        'slug.required'        => 'O slug é obrigatório',
        'description.required' => 'A descrição é obrigatória',
        'date.required'        => 'A data é obrigatória',
    );

    /**
     * Fillable fields
     */
    protected $fillable = array(
        'news_category_id',
        'title',
        'subtitle',
        'description',
        'featured',
        'active',
        'date',
        'source',
    );

    /**
     * Setando a data
     */
    public function setDateAttribute($value)
    {
        try {
            $this->attributes['date'] = Carbon::createFromFormat('d/m/Y', $value);
        } catch (InvalidArgumentException $e) {
            $this->attributes['date'] = null;
        }
    }

    /**
     * Setando o autor
     */
    public function setSourceAttribute($value)
    {
        $this->attributes['source'] = $value != '' ? $value : null;
    }

    /**
     * Medias relashionship
     */
    public function medias()
    {
        return $this->morphMany('EscapeWork\Manager\Medias\Models\Media', 'medias', 'media_model', 'model_id')
                    ->orderBy('order');
    }

    /**
     * Escopo para noticias ativas
     */
    public function scopeActive($query)
    {
        return $query->where('active', '=', 1);
    }

    /**
     * Escopo para noticias ativas
     */
    public function scopeFeatured($query)
    {
        return $query->where('featured', '=', 1);
    }

    /**
     * Escopo para fazer join com a tabela de categorias
     */
    public function scopeJoinCategories($query)
    {
        return $query->join('news_categories', 'news_categories.id', '=', 'news.news_category_id');
    }

    /**
     * Escopo para fazer left join com a tabela de categorias
     */
    public function scopeLeftJoinCategories($query)
    {
        return $query->leftJoin('news_categories', 'news_categories.id', '=', 'news.news_category_id');
    }

    /**
     * Retornando o model atual (esta função pode ser sobreescrita)
     */
    public function getMediaModel()
    {
        return 'EscapeWork\Manager\News\Models\News';
    }

    /**
     * Excluindo uma notícia e suas mídias
     */
    public function delete()
    {
        $this->deleteMedias();

        return parent::delete();
    }

    /**
     * Excluindo as midias
     */
    protected function deleteMedias()
    {
        foreach ($this->medias as $media) {
            $media->delete();
        }
    }
}
